import { Component, inject, OnDestroy, signal } from '@angular/core';
import RedemptionService from '../../shared/services/redemption.service';
import Redemption from '../../shared/models/redemption';
import { MatCardModule } from '@angular/material/card';
import { TwitchRedemptionDropdownComponent } from "../twitch-redemption-dropdown/twitch-redemption-dropdown.component";
import { MatFormFieldModule } from '@angular/material/form-field';
import { ActivatedRoute } from '@angular/router';
import { ActionDropdownComponent } from '../../actions/action-dropdown/action-dropdown.component';
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { MatInputModule } from '@angular/material/input';
import { MatTableModule } from '@angular/material/table';
import TwitchRedemption from '../../shared/models/twitch-redemption';
import { RedemptionItemEditComponent } from '../redemption-item-edit/redemption-item-edit.component';
import { MatDialog } from '@angular/material/dialog';
import RedeemableAction from '../../shared/models/redeemable-action';
import { MatExpansionModule } from '@angular/material/expansion';
import { ScrollingModule } from '@angular/cdk/scrolling';
import { Subscription } from 'rxjs';
import { HermesClientService } from '../../hermes-client.service';

@Component({
  selector: 'redemption-list',
  imports: [
    ActionDropdownComponent,
    MatButtonModule,
    MatCardModule,
    MatExpansionModule,
    MatFormFieldModule,
    MatIconModule,
    MatInputModule,
    MatTableModule,
    ScrollingModule,
    TwitchRedemptionDropdownComponent,
  ],
  templateUrl: './redemption-list.component.html',
  styleUrl: './redemption-list.component.scss'
})
export class RedemptionListComponent implements OnDestroy {
  private readonly client = inject(HermesClientService);
  private readonly redemptionService = inject(RedemptionService);
  private readonly route = inject(ActivatedRoute);
  readonly dialog = inject(MatDialog);
  private _redemptions: Redemption[] = [];
  private _twitchRedemptions: TwitchRedemption[] = [];
  private _twitchRedemptionsDict: { [id: string]: string } = {};
  private _actions: RedeemableAction[] = [];
  displayedColumns: string[] = ['twitch-redemption', 'action-name', 'order', 'misc'];
  filter_redemption: string | undefined;
  filter_action_name: string | undefined;
  readonly panelOpenState = signal(false);
  private _subscriptions: Subscription[] = []


  constructor() {
    this.route.data.subscribe(r => {
      this._twitchRedemptions = r['twitchRedemptions'];
      this._twitchRedemptionsDict = Object.assign({}, ...r['twitchRedemptions'].map((t: TwitchRedemption) => ({ [t.id]: t.title })));
      this._actions = r['redeemableActions'];

      let redemptions = [...r['redemptions']];
      redemptions.sort((a: Redemption, b: Redemption) => this.compare(a, b));
      this._redemptions = redemptions;
    });

    let subscription = this.redemptionService.create$?.subscribe(d => {
      if (d.error || !d.data || d.request.nounce != null && d.request.nounce.startsWith(this.client.session_id))
        return;

      let index = -1;
      for (let i = 0; i < this._redemptions.length; i++) {
        const comp = this.compare(d.data, this._redemptions[i]);
        if (comp < 0) {
          index = i;
          break;
        }
      }
      this._redemptions.splice(index >= 0 ? index : this._redemptions.length, 0, d.data);
    });
    if (subscription)
      this._subscriptions.push(subscription);

    subscription = this.redemptionService.update$?.subscribe(d => {
      if (d.error || !d.data || d.request.nounce != null && d.request.nounce.startsWith(this.client.session_id))
        return;

      const redemption = this._redemptions.find(r => r.id = d.data.id);
      if (redemption) {
        redemption.action_name = d.data.action_name;
        redemption.redemption_id = d.data.redemption_id;
        redemption.order = d.data.order;
        redemption.state = d.data.state;
      }
    });
    if (subscription)
      this._subscriptions.push(subscription);

    subscription = this.redemptionService.delete$?.subscribe(d => {
      if (d.error || d.request.nounce != null && d.request.nounce.startsWith(this.client.session_id))
        return;

      this._redemptions = this._redemptions.filter(r => r.id != d.request.data.id);
    });
    if (subscription)
      this._subscriptions.push(subscription);
  }

  ngOnInit() {
    this.panelOpenState.set(false);
  }

  ngOnDestroy(): void {
    this._subscriptions.forEach(s => s.unsubscribe());
  }

  compare(a: Redemption, b: Redemption) {
    return this._twitchRedemptionsDict[a.redemption_id].localeCompare(this._twitchRedemptionsDict[b.redemption_id]) || a.order - b.order;
  }

  get redemptions() {
    const redemptionFilter = this.filter_redemption?.toString().toLowerCase();
    const actionFilter = this.filter_action_name?.toString().toLowerCase();
    let filtered = this._redemptions.filter(r => !redemptionFilter || this._twitchRedemptionsDict[r.redemption_id].toLowerCase().includes(redemptionFilter));
    filtered = filtered.filter(r => !actionFilter || r.action_name.toLowerCase().includes(actionFilter));
    return filtered;
  }

  getTwitchRedemptionNameById(id: string) {
    return this._twitchRedemptionsDict[id];
  }

  add(): void {
    this.openDialog({ id: '', user_id: '', redemption_id: '', action_name: '', order: 0, state: true });
  }

  openDialog(redemption: Redemption): void {
    const dialogRef = this.dialog.open(RedemptionItemEditComponent, {
      data: { redemption: { ...redemption }, twitchRedemptions: this._twitchRedemptions, redeemableActions: this._actions },
      maxWidth: '100vw'
    });

    dialogRef.afterClosed().subscribe((result: Redemption | string) => {
      if (!result)
        return;

      if (typeof result === 'string') {
        // Deleted
        this._redemptions = this._redemptions.filter(r => r.id != result);
      } else {
        const redemption = this._redemptions.find(r => r.id == result.id);
        if (redemption) {
          // Updated
          redemption.action_name = result.action_name;
          redemption.redemption_id = result.redemption_id;
          redemption.order = result.order;
          redemption.state = result.state;
        } else {
          // Created
          let index = -1;
          for (let i = 0; i < this._redemptions.length; i++) {
            const comp = this.compare(result, this._redemptions[i]);
            if (comp < 0) {
              index = i;
              break;
            }
          }
          this._redemptions.splice(index >= 0 ? index : this._redemptions.length, 0, result);
        }
      }
    });
  }
}